@@ -3,6 +3,8 @@ package HTML::Linear;
 use strict;
 use common::sense;
 
+use Digest::SHA qw(sha256);
+
 use Any::Moose;
 use Any::Moose qw(X::NonMoose);
 extends 'HTML::TreeBuilder';
@@ -10,7 +12,7 @@ extends 'HTML::TreeBuilder';
 use HTML::Linear::Element;
 use HTML::Linear::Path;
 
-our $VERSION = '0.006'; # VERSION
+our $VERSION = '0.007'; # VERSION
 
 
 has _list       => (
@@ -19,7 +21,7 @@ has _list       => (
     isa         => 'ArrayRef[Any]',
     default     => sub { [] },
     handles     => {
-        add_element     => 'push',
+        _add_element    => 'push',
         as_list         => 'elements',
         count_elements  => 'count',
         get_element     => 'accessor',
@@ -27,6 +29,18 @@ has _list       => (
 );
 
 
+has _shrink => (
+    traits      => ['Bool'],
+    is          => 'ro',
+    isa         => 'Bool',
+    default     => 0,
+    handles     => {
+        set_shrink      => 'set',
+        unset_shrink    => 'unset',
+    },
+);
+
+
 has _strict => (
     traits      => ['Bool'],
     is          => 'ro',
@@ -42,20 +56,53 @@ has _strict => (
 has _uniq       => (is => 'ro', isa => 'HashRef[Str]', default => sub { {} });
 
 
+has _path_count => (is => 'ro', isa => 'HashRef[Str]', default => sub { {} });
+
+
 after eof => sub {
     my ($self) = @_;
 
     $self->deparse($self, []);
 
-    my $i = 0;
-    my %uniq;
-    for my $elem ($self->as_list) {
-        $elem->index($uniq{join ',', $elem->path}++);
-        $elem->index_map($self->_uniq);
+    if ($self->_shrink) {
+        my %short;
+        for my $elem ($self->as_list) {
+            my @rpath = reverse $elem->as_xpath;
+            my $i = 0;
+            unless ($self->_strict) {
+                for (; $i <= $#rpath; $i++) {
+                    last if $elem->path->[-1 - $i]->is_groupable;
+                }
+            }
+            for my $j ($i .. $#rpath) {
+                my $key = sha256(join '' => @rpath[0 .. $j]);
+                $short{$key}{offset} = $#rpath - $j;
+                push @{$short{$key}{elem}}, $elem;
+                ++$short{$key}{accumulator}{$elem->as_xpath};
+            }
+        }
+
+        for my $key (sort { $short{$b}{offset} <=> $short{$a}{offset} } keys %short) {
+            next if 1 < keys %{$short{$key}{accumulator}};
+            for my $elem (@{$short{$key}{elem}}) {
+                next if $elem->trim_at;
+                $elem->trim_at($short{$key}{offset});
+            }
+        }
     }
 };
 
 
+sub add_element {
+    my ($self, $elem) = @_;
+
+    $elem->index($self->_path_count->{join ',', $elem->path}++);
+    $elem->index_map($self->_uniq);
+
+    $self->_add_element($elem);
+}
+
+
 sub deparse {
     my ($self, $node, $path) = @_;
 
@@ -78,6 +125,7 @@ sub deparse {
             HTML::Linear::Element->new({
                 depth   => $node->depth,
                 path    => [ @{$path}, $level ],
+                strict  => $self->_strict,
             })
         );
     }
@@ -86,13 +134,14 @@ sub deparse {
     for my $child ($node->content_list) {
         if (ref $child) {
             my $l = $self->deparse($child, [ @{$path}, $level ]);
-            push @{$uniq{$l->as_xpath}}, $l->address;
+            push @{$uniq{$l->as_xpath(1)}}, $l->address;
         } else {
             $self->add_element(
                 HTML::Linear::Element->new({
                     content => $child,
                     depth   => $node->depth,
                     path    => [ @{$path}, $level ],
+                    strict  => $self->_strict,
                 })
             );
         }
@@ -128,7 +177,7 @@ HTML::Linear - represent HTML::Tree as a flat list
 
 =head1 VERSION
 
-version 0.006
+version 0.007
 
 =head1 SYNOPSIS
 
@@ -149,6 +198,10 @@ version 0.006
 
 Internal list representation.
 
+=head2 _shrink
+
+Internal shrink mode flag.
+
 =head2 _strict
 
 Internal strict mode flag.
@@ -157,12 +210,12 @@ Internal strict mode flag.
 
 Used for internal collision detection.
 
+=head2 _path_count
+
+Used internally for computing numeric tag indexes (like in C</p[3]>).
+
 =head1 METHODS
 
-=head2 add_element
-
-Add an element to the list.
-
 =head2 as_list
 
 Access list as array.
@@ -175,6 +228,14 @@ Number of elements in list.
 
 Element accessor.
 
+=head2 set_shrink
+
+Enable XPath shrinking.
+
+=head2 unset_shrink
+
+Disable XPath shrinking.
+
 =head2 set_strict
 
 Do not group by C<id>, C<class> or C<name> attributes.
@@ -187,6 +248,10 @@ Group by C<id>, C<class> or C<name> attributes.
 
 Overrides L<HTML::TreeBuilder> C<eof>.
 
+=head2 add_element
+
+Add an element to the list.
+
 =head2 deparse($node, $path)
 
 Recursively scan underlying L<HTML::TreeBuilder> structure.
