@@ -9,7 +9,7 @@ use Any::Moose;
 
 use HTML::Linear::Path;
 
-our $VERSION = '0.006'; # VERSION
+our $VERSION = '0.007'; # VERSION
 
 
 has attributes  => (is => 'rw', isa => 'HashRef[Str]', default => sub { {} }, auto_deref => 1);
@@ -20,6 +20,8 @@ has index_map   => (is => 'rw', isa => 'HashRef[Str]', default => sub { {} }, au
 has key         => (is => 'rw', isa => 'Str', default => '');
 has path        => (is => 'ro', isa => 'ArrayRef[HTML::Linear::Path]', required => 1, auto_deref => 1);
 has sha         => (is => 'ro', isa => 'Digest::SHA', default => sub { new Digest::SHA(256) }, lazy => 1 );
+has strict      => (is => 'ro', isa => 'Bool', default => 0);
+has trim_at     => (is => 'rw', isa => 'Int', default => 0);
 
 use overload '""' => \&as_string, fallback => 1;
 
@@ -44,11 +46,17 @@ sub as_string {
 
 sub as_xpath {
     my ($self) = @_;
-    return
-        join '',
-            map {
-                $_->as_xpath . ($self->index_map->{$_->address} // '')
-            } $self->path;
+    my @xpath = map {
+        $_->as_xpath . (
+            (not $self->strict and $_->is_groupable)
+                ? ''
+                : $self->index_map->{$_->address} // ''
+            )
+    } ($self->path) [$self->trim_at .. $#{$self->path}];
+    $self->trim_at and unshift @xpath, HTML::Linear::Path::_wrap(separator => '/');
+    return wantarray
+        ? @xpath
+        : join '', @xpath;
 }
 
 
@@ -63,7 +71,13 @@ sub as_hash {
             . HTML::Linear::Path::_wrap(sigil       => '@')
             . HTML::Linear::Path::_wrap(attribute   => $key)
         } = $self->attributes->{$key}
-            unless $self->attributes->{$key} =~ m{^\s*$}s;
+            if
+                $self->attributes->{$key} !~ m{^\s*$}s
+                and not (
+                    $self->strict
+                        ? 0
+                        : HTML::Linear::Path::_isgroup($self->path->[-1]->tag, $key)
+                );
     }
 
     $hash->{
@@ -96,7 +110,7 @@ HTML::Linear::Element - represent elements to populate HTML::Linear
 
 =head1 VERSION
 
-version 0.006
+version 0.007
 
 =head1 SYNOPSIS
 
@@ -142,6 +156,14 @@ Store representations of paths inside C<HTML::TreeBuilder> structure (L<HTML::Li
 
 Lazy L<Digest::SHA> (256-bit) representation.
 
+=head2 strict
+
+Strict mode disables grouping by tags/attributes listed in L<HTML::Linear::Path/%HTML::Linear::Path::groupby>.
+
+=head2 trim_at
+
+XPath seems to be unique after that level.
+
 =head1 METHODS
 
 =head2 as_string
@@ -152,6 +174,8 @@ Stringified signature of an element.
 
 Build a nice XPath representation of a path inside the L<HTML::TreeBuilder> structure.
 
+Returns string in scalar context or XPath segments in list context.
+
 =head2 as_hash
 
 Linearize element as an associative array (Perl hash).
